@@ -1,6 +1,6 @@
 ;;; meson-mode.el --- Major mode for the Meson build system files  -*- lexical-binding: t; -*-
 
-;; Copyright (C) 2017, 2020  Michal Sojka
+;; Copyright (C) 2017, 2020, 2021  Michal Sojka
 
 ;; Author: Michal Sojka <sojkam1@fel.cvut.cz>
 ;; Version: 0.2
@@ -115,6 +115,8 @@ If some arguments are numbers, threat them as string lengths."
       ("install_headers" :doc "void install_headers(list_of_headers, ...)")
       ("install_man" :doc "void install_man(list_of_manpages, ...)")
       ("install_subdir" :doc "void install_subdir(subdir_name, install_dir : ..., exclude_files : ..., exclude_directories : ..., strip_directory : ...)")
+      ("install_emptydir" :doc "void install_emptydir(list_of_dirs, ...)")
+      ("install_symlink" :doc "void install_symlink(link_name, ...)")
       ("is_disabler" :doc "bool is_disabler(var)")
       ("is_variable" :doc "bool is_variable(varname)")
       ("jar" :doc nil)
@@ -486,6 +488,10 @@ If some arguments are numbers, threat them as string lengths."
      . ("install_dir" "install_mode"))
     ("install_subdir"
      . ("exclude_files" "exclude_directories" "install_dir" "install_mode" "strip_directory"))
+    ("install_emptydir"
+     . ("install_mode" "install_tag"))
+    ("install_symlink"
+     . ("install_dir" "install_tag" "pointing_to"))
     ("jar"
      . ,meson--known-jar-kwargs)
     ("project"
@@ -603,7 +609,7 @@ The point can be anywhere within function name or argument list."
 	    (nth 4 ppss))		; inside comment
 	(goto-char (nth 8 ppss))	; go to the beginning
 	(meson-function-at-point))
-       ((cl-some (lambda (fname) (when (looking-at fname) fname)) functions))
+       ((cl-some (lambda (fname) (when (looking-at-p fname) fname)) functions))
        ((and (> (nth 0 ppss) 0)			 ; inside parentheses
 	     (eq (char-after (nth 1 ppss)) ?\()) ; rounded parentheses
 	(goto-char (nth 1 ppss))
@@ -682,7 +688,7 @@ Optional SYN-PPSS is the value returned by `syntax-ppss'."
 	;; that we can still return "eol" token there.
 	(when (or (nth 4 ppss)
 		  (and (not (nth 3 ppss)) ; not inside string
-		       (looking-at "#")))
+		       (looking-at-p "#")))
 	  (end-of-line)
 	  (setq ppss (syntax-ppss)))	; update ppss after move
 	;; Determine token but do not move behind it
@@ -865,6 +871,11 @@ Optional SYN-PPSS is the value returned by `syntax-ppss'."
   "Directory containing Meson markdown-formated documentation."
   :type 'directory)
 
+(defcustom meson-doc-display-buffer-action
+  '((display-buffer-reuse-window display-buffer-same-window))
+  "The display action used, when displaying Meson documentation."
+  :type display-buffer--action-custom-type)
+
 (defun meson-smie-rules (kind token)
   "Indentation rules for the SMIE engine.
 See the SMIE documentation for the meaning of KIND and TOKEN
@@ -882,13 +893,12 @@ arguments."
     (`(:list-intro . ,(or "eol" ":" "")) t) ; "" is actually "[" because that's what lexer returns
     (`(:after . ":") meson-indent-basic)
     (`(:after . ,(or "=" "+=")) meson-indent-basic)
-    (`(:before . "[") (if (smie-rule-hanging-p) (smie-rule-parent)))
-    (`(:before . "(") (if (smie-rule-hanging-p)
-			  (save-excursion
-			    (smie-backward-sexp 'halfsexp) ; goto parent
-			    (beginning-of-line-text)
-			    (cons 'column (current-column)))))
-    (`(:after . ,(or "[" "(")) meson-indent-basic)
+    (`(:before . ,(or "[" "(" "{")) (if (smie-rule-hanging-p)
+					(save-excursion
+					  (smie-backward-sexp 'halfsexp) ; goto parent
+					  (beginning-of-line-text)
+					  (cons 'column (current-column)))))
+    (`(:after . ,(or "[" "(" "{")) meson-indent-basic)
     (`(:before . "elif") (smie-rule-parent))
     (_ nil)))
 
@@ -945,12 +955,15 @@ or does not contain IDENTIFIER."
           (markdown-view-mode))
         (local-set-key (kbd "q") 'bury-buffer)
         (when (bound-and-true-p evil-mode)
+          (declare-function evil-local-set-key nil) ;; suppress "undeclared" warning
           (evil-local-set-key 'normal (kbd "q") 'bury-buffer)))
-      (goto-char
-       (or (meson--search-in-reference-manual identifier)
-           (user-error "%s not found in Meson reference manual" identifier))))
-    (switch-to-buffer buf)
-    (recenter 0)
+      (let* ((position
+	      (or (meson--search-in-reference-manual identifier)
+		  (user-error "%s not found in Meson reference manual" identifier)))
+	     (window (display-buffer buf meson-doc-display-buffer-action)))
+	(with-selected-window window
+	  (goto-char position)
+	  (recenter 0))))
     buf))
 
 (defalias 'meson-lookup-doc-at-point (symbol-function 'meson-lookup-doc))
@@ -977,6 +990,7 @@ or does not contain IDENTIFIER."
 
   (set (make-local-variable 'comment-start) "# ")
   (set (make-local-variable 'comment-end) "")
+  (setq indent-tabs-mode nil)
   (add-hook 'completion-at-point-functions
             #'meson-completion-at-point-function nil t)
   (smie-setup meson-smie-grammar #'meson-smie-rules
